unit OpenWithEditorModule;

interface

uses
	Windows, SysUtils, ShellApi, ActiveX,
	{$IFNDEF EXTENSION}LObjects, {$ENDIF} EOFileIntf, DYAYaml;

type
	TForceEditor = class(TInterfacedObject, IAction, IObserver)
	private
		FEnabled: Boolean;
		{IActionItem}
		function GetText: WideString;
		function GetChecked: Boolean;
		function GetItems: IActionItems;
		function Execute: Boolean;
		{IAction}
		function GetName: string;
		function GetHint: WideString;
		function GetEnabled: Boolean;
		{IObserver}
		procedure Observe;
	end;

	TGlobalExtension = class(TInterfacedObject, IExtension, IExtensionInfo, IThreadHandler)
	private
		{IExtension}
		procedure Initialize;
		procedure Finalize;
		{IExtensionInfo}
		function GetName: WideString;
		procedure Dialog;
		{IThreadHandler}
		procedure ThreadBegun(const AThread: IInterface);
		procedure ThreadEnded(const AThread: IInterface);
	end;

implementation

function WideLastDelimiter(const Delimiters, S: WideString): Integer;
begin
	Result := Length(S);
	while Result > 0 do
	begin
		if Pos(S[Result], Delimiters) > 0 then Break;
		Dec(Result);
	end;
end;

function WideExtractFilePath(const FileName: WideString): WideString;
var
	I: Integer;
begin
	I := WideLastDelimiter(PathDelim + DriveDelim, FileName);
	Result := Copy(FileName, 1, I);
end;

function WideIncludeTrailingPathDelimiter(const S: WideString): WideString;
begin
	Result := S;
	if (Result = '') or (Result[Length(Result)] <> PathDelim) then
		Result := Result + PathDelim;
end;

{ TForceEditor }

type
	TContext = record
		Reader: IYamlSerializer;
		Result: WideString;
	end;

procedure ReadRootHash(Context: Pointer); cdecl;
begin
	IO(TContext(Context^).Reader, 'editor', TContext(Context^).Result)
end;

function TForceEditor.Execute: Boolean;

	function ReadSetting: WideString;  
	var
		Context: TContext;
	begin
		Context.Reader := (GetApplication as IYamlSerializer);
		Context.Result := '';
		IO(Context.Reader, ReadRootHash, @Context);
		Result := Context.Result;
	end;

var
	Param, Editor: WideString;
	View: IView;
begin
	View := (GetCurrentThread as IEOFileThread).ActiveView;
	with View.FocusedFile^ do SetString(Param, Name, NameLength);
	if Pos(':', Param) = 0 then
		Param :=  WideIncludeTrailingPathDelimiter(View.Model.Path.Directory) + Param;
	Param := '"' + Param + '"';

	Editor := ReadSetting;
	if Editor = '' then Editor := 'notepad.exe';

	ShellExecuteW(0, nil, PWideChar(Editor), PWideChar(Param), PWideChar(View.Model.Path.Directory), SW_SHOW);

	Result := True
end;

function TForceEditor.GetChecked: Boolean;
begin
	Result := False
end;

function TForceEditor.GetEnabled: Boolean;
begin
	Result := FEnabled
end;

function TForceEditor.GetHint: WideString;
begin
	Result := 'GfB^ŋIɊJ܂'
end;

function TForceEditor.GetItems: IActionItems;
begin
	Result := nil
end;

function TForceEditor.GetName: string;
begin
	Result := 'OpenWithEditor'
end;

function TForceEditor.GetText: WideString;
begin
	Result := 'GfB^ŊJ'
end;

procedure TForceEditor.Observe;
var
	View: IView;

	function SelectedIsRealFiles: Boolean;
	begin
		if Supports(View.Model, IArchive) then
		begin
			Result := False;
			Exit
		end;

		if View.SelectedCount = 0 then
		begin
			Result := (View.Count > 0) and (View.FocusedFile.Kind = fkFile)
		end
		else
			Result := False;
	end;

begin
	View := (GetCurrentThread as IEOFileThread).ActiveView;
	FEnabled := SelectedIsRealFiles;
end;

{ TGlobalExtension }

procedure TGlobalExtension.Dialog;
begin
	MessageBoxW(0, 'by YT', 'GfB^ŊJ', MB_ICONINFORMATION)
end;

procedure TGlobalExtension.Finalize;
begin
end;

function TGlobalExtension.GetName: WideString;
begin
	Result := 'Force Editor'
end;

procedure TGlobalExtension.Initialize;
begin
end;

procedure TGlobalExtension.ThreadBegun(const AThread: IInterface);
var
	Action: IAction;
begin
	Action := TForceEditor.Create;

	//Xbh[JANVo^
	(AThread as IActionManager).Attach(Action);

	//Ďo^
	(AThread as IObservable).Attach(Action as IObserver);
end;

procedure TGlobalExtension.ThreadEnded(const AThread: IInterface);
begin
end;

procedure Register;
begin
	(GetApplication as IExtensionManager).RegisterExtension(TGlobalExtension.Create);
end;

initialization

	if GetInterfaceVersion = EOFileIntf.InterfaceVersion then
		Register;

end.
